﻿using System;
using Silanis.ESL.SDK;
using Silanis.ESL.SDK.Builder;
using System.Collections.Generic;
using System.Diagnostics;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.IO;
using Newtonsoft.Json.Linq;

namespace FeatureGuideCharp
{
    class EnforceCaptureSignatureREST
    {
        private string API_KEY = "api_key";
        private string API_URL = "https://sandbox.esignlive.com/api";
        private string FILE_PATH = "file_path";

        public void Execute()
        {
            
            // Prepare the client
            HttpClient httpClient = new HttpClient();
            httpClient.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Basic", API_KEY);
            httpClient.DefaultRequestHeaders.Add("Accept", "application/json");

            // Build the request
            StringBuilder sb = new StringBuilder();


            sb.AppendLine("            {                                                ");
            sb.AppendLine("  \"documents\": [                                           ");
            sb.AppendLine("    {                                                        ");
            sb.AppendLine("      \"approvals\": [                                       ");
            sb.AppendLine("        {                                                    ");
            sb.AppendLine("          \"fields\": [                                      ");
            sb.AppendLine("            {                                                ");
            sb.AppendLine("              \"height\": 50,                                ");
            sb.AppendLine("              \"id\": \"Signature1\",                        ");
            sb.AppendLine("              \"left\": 100,                                 ");
            sb.AppendLine("              \"page\": 0,                                   ");
            sb.AppendLine("              \"subtype\": \"CAPTURE\",                      ");
            sb.AppendLine("              \"top\": 100,                                  ");
            sb.AppendLine("              \"type\": \"SIGNATURE\",                       ");
            sb.AppendLine("              \"width\": 200                                 ");
            sb.AppendLine("            }                                                ");
            sb.AppendLine("          ],                                                 ");
            sb.AppendLine("          \"id\": \"Signature1\",                            ");
            sb.AppendLine("          \"role\": \"signer1\"                              ");
            sb.AppendLine("        },                                                   ");
            sb.AppendLine("        {                                                    ");
            sb.AppendLine("          \"fields\": [                                      ");
            sb.AppendLine("            {                                                ");
            sb.AppendLine("              \"height\": 50,                                ");
            sb.AppendLine("              \"id\": \"Signature2\",                        ");
            sb.AppendLine("              \"left\": 100,                                 ");
            sb.AppendLine("              \"page\": 0,                                   ");
            sb.AppendLine("              \"subtype\": \"CAPTURE\",                      ");
            sb.AppendLine("              \"top\": 200,                                  ");
            sb.AppendLine("              \"type\": \"SIGNATURE\",                       ");
            sb.AppendLine("              \"width\": 200                                 ");
            sb.AppendLine("            }                                                ");
            sb.AppendLine("          ],                                                 ");
            sb.AppendLine("          \"id\": \"Signature2\",                            ");
            sb.AppendLine("          \"role\": \"signer1\",                             ");
            sb.AppendLine("          \"enforceCaptureSignature\": true                  ");
            sb.AppendLine("        },                                                   ");
            sb.AppendLine("        {                                                    ");
            sb.AppendLine("          \"fields\": [                                      ");
            sb.AppendLine("            {                                                ");
            sb.AppendLine("              \"height\": 50,                                ");
            sb.AppendLine("              \"id\": \"Signature3\",                        ");
            sb.AppendLine("              \"left\": 100,                                 ");
            sb.AppendLine("              \"page\": 0,                                   ");
            sb.AppendLine("              \"subtype\": \"CAPTURE\",                      ");
            sb.AppendLine("              \"top\": 300,                                  ");
            sb.AppendLine("              \"type\": \"SIGNATURE\",                       ");
            sb.AppendLine("              \"width\": 200                                 ");
            sb.AppendLine("            }                                                ");
            sb.AppendLine("          ],                                                 ");
            sb.AppendLine("          \"id\": \"Signature3\",                            ");
            sb.AppendLine("          \"role\": \"signer1\"                              ");
            sb.AppendLine("        }                                                    ");
            sb.AppendLine("      ],                                                     ");
            sb.AppendLine("      \"name\": \"Document1\",                               ");
            sb.AppendLine("      \"id\": \"Document1\"                                  ");
            sb.AppendLine("    }                                                        ");
            sb.AppendLine("  ],                                                         ");
            sb.AppendLine("  \"name\": \"test Enforce Capture at Signature Level\",     ");
            sb.AppendLine("  \"roles\": [                                               ");
            sb.AppendLine("    {                                                        ");
            sb.AppendLine("      \"id\": \"signer1\",                                   ");
            sb.AppendLine("      \"signers\": [                                         ");
            sb.AppendLine("        {                                                    ");
            sb.AppendLine("          \"email\": \"signer1@mailinator.com\",             ");
            sb.AppendLine("          \"firstName\": \"John\",                           ");
            sb.AppendLine("          \"id\": \"signer1\",                               ");
            sb.AppendLine("          \"lastName\": \"Smith\"                            ");
            sb.AppendLine("        }                                                    ");
            sb.AppendLine("      ]                                                      ");
            sb.AppendLine("    }                                                        ");
            sb.AppendLine("  ],                                                         ");
            sb.AppendLine("  \"type\": \"PACKAGE\",                                     ");
            sb.AppendLine("  \"status\": \"SENT\"                                       ");
            sb.AppendLine("}                                                            ");

            StringContent requestContent = new StringContent(sb.ToString(), Encoding.UTF8, "application/json");

            // File upload
            byte[] fileByteArray = File.ReadAllBytes(FILE_PATH);
            ByteArrayContent byteArrayContent = new ByteArrayContent(fileByteArray);
            byteArrayContent.Headers.ContentDisposition = new ContentDispositionHeaderValue("form-data");
            byteArrayContent.Headers.ContentDisposition.Name = "\"file\"";
            byteArrayContent.Headers.ContentDisposition.FileName = "\"Document 1\"";
            byteArrayContent.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

            MultipartFormDataContent form = new MultipartFormDataContent();
            form.Add(byteArrayContent, "\"file\"", "\"Document 1\"");
            form.Add(requestContent, "\"payload\"");


            // Create/Send Package
            var response = httpClient.PostAsync(new Uri(API_URL) + "/packages/", form).Result;
            if (response.StatusCode == System.Net.HttpStatusCode.OK)
            {
                var stringResult = response.Content.ReadAsStringAsync().Result;
                var jsonResult = JObject.Parse(stringResult);
                JToken packageId = jsonResult["id"];
                Debug.WriteLine("Package Id: " + packageId);
            }
            else {
                Debug.WriteLine("Error Message: " + response.Content.ReadAsStringAsync().Result);
            }

        }
    }
}
